@@ -1,4 +1,19 @@
-/*
+/*  Copyright 1997,2001-2003 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ *
  * Buffer read/write module
  */
 
@@ -18,16 +33,33 @@ typedef struct Buffer_t {
 
 	size_t sectorSize;	/* sector size: all operations happen
 				 * in multiples of this */
-	size_t cylinderSize;	/* cylinder size: preferred alignemnt,
+	size_t cylinderSize;	/* cylinder size: preferred alignment,
 				 * but for efficiency, less data may be read */
 	int ever_dirty;	       	/* was the buffer ever dirty? */
 	size_t dirty_pos;
 	size_t dirty_end;
-	mt_off_t current;		/* first sector in buffer */
-	size_t cur_size;		/* the current size */
+	mt_off_t current;	/* first sector in buffer */
+	size_t cur_size;	/* the current size */
 	char *buf;		/* disk read/write buffer */
 } Buffer_t;
 
+/* Convert position relative to buffer to absolute position */
+static mt_off_t abs_pos(Buffer_t *Buffer, size_t rel) {
+	return Buffer->current + (mt_off_t) rel;
+}
+
+/* End of currently valid buffer */
+static mt_off_t cur_end(Buffer_t *Buffer) {
+	return abs_pos(Buffer, Buffer->cur_size);
+}
+
+/* distance from absolute position until next full cylinder. If position already
+ * *is* on a full cylinder boundary, return size of full cylinder */
+static size_t pos_to_next_full_cyl(Buffer_t *Buffer, mt_off_t pos) {
+	return Buffer->cylinderSize -
+		(size_t) (pos % (mt_off_t) Buffer->cylinderSize);
+}
+
 /*
  * Flush a dirty buffer to disk.  Resets Buffer->dirty to zero.
  * All errors are fatal.
@@ -35,14 +67,10 @@ typedef struct Buffer_t {
 
 static int _buf_flush(Buffer_t *Buffer)
 {
-	int ret;
+	ssize_t ret;
 
 	if (!Buffer->Next || !Buffer->dirty)
 		return 0;
-	if(Buffer->current < 0L) {
-		fprintf(stderr,"Should not happen\n");
-		return -1;
-	}
 #ifdef DEBUG
 	fprintf(stderr, "write %08x -- %02x %08x %08x\n",
 		Buffer,
@@ -51,9 +79,9 @@ static int _buf_flush(Buffer_t *Buffer)
 		Buffer->dirty_end - Buffer->dirty_pos);
 #endif
 
-	ret = force_write(Buffer->Next, 
+	ret = force_write(Buffer->Next,
 			  Buffer->buf + Buffer->dirty_pos,
-			  Buffer->current + Buffer->dirty_pos,
+			  Buffer->current + (mt_off_t) Buffer->dirty_pos,
 			  Buffer->dirty_end - Buffer->dirty_pos);
 	if(ret != (signed int) (Buffer->dirty_end - Buffer->dirty_pos)) {
 		if(ret < 0)
@@ -70,39 +98,33 @@ static int _buf_flush(Buffer_t *Buffer)
 
 static int invalidate_buffer(Buffer_t *Buffer, mt_off_t start)
 {
-	/*fprintf(stderr, "invalidate %x\n", Buffer);*/
-	if(Buffer->sectorSize == 32) {
-		fprintf(stderr, "refreshing directory\n");
-	}
-
 	if(_buf_flush(Buffer) < 0)
 		return -1;
 
 	/* start reading at the beginning of start's sector
 	 * don't start reading too early, or we might not even reach
 	 * start */
-	Buffer->current = ROUND_DOWN(start, Buffer->sectorSize);
+	Buffer->current = ROUND_DOWN(start, (mt_off_t) Buffer->sectorSize);
 	Buffer->cur_size = 0;
 	return 0;
 }
 
 #undef OFFSET
-#define OFFSET (start - This->current)
+#define OFFSET ((size_t)(start - This->current))
 
 typedef enum position_t {
 	OUTSIDE,
 	APPEND,
 	INSIDE,
-	ERROR 
+	ERROR
 } position_t;
 
 static position_t isInBuffer(Buffer_t *This, mt_off_t start, size_t *len)
 {
-	if(start >= This->current &&
-	   start < This->current + This->cur_size) {
+	if(start >= This->current && start < cur_end(This)) {
 		maximize(*len, This->cur_size - OFFSET);
 		return INSIDE;
-	} else if(start == This->current + This->cur_size &&
+	} else if(start == cur_end(This) &&
 		  This->cur_size < This->size &&
 		  *len >= This->sectorSize) {
 		/* append to the buffer for this, three conditions have to
@@ -119,17 +141,17 @@ static position_t isInBuffer(Buffer_t *This, mt_off_t start, size_t *len)
 		if(invalidate_buffer(This, start) < 0)
 			return ERROR;
 		maximize(*len, This->cylinderSize - OFFSET);
-		maximize(*len, This->cylinderSize - This->current % This->cylinderSize);
+		maximize(*len, pos_to_next_full_cyl(This, This->current));
 		return OUTSIDE;
 	}
 }
 
-static int buf_read(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
+static ssize_t buf_read(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 {
 	size_t length;
-	int offset;
+	size_t offset;
 	char *disk_ptr;
-	int ret;
+	ssize_t ret;
 	DeclareThis(Buffer_t);	
 
 	if(!len)
@@ -140,19 +162,18 @@ static int buf_read(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 		case OUTSIDE:
 		case APPEND:
 			/* always load until the end of the cylinder */
-			length = This->cylinderSize -
-				(This->current + This->cur_size) % This->cylinderSize;
+			length = pos_to_next_full_cyl(This, cur_end(This));
 			maximize(length, This->size - This->cur_size);
 
 			/* read it! */
 			ret=READS(This->Next,
 				  This->buf + This->cur_size,
-				  This->current + This->cur_size,
+				  This->current + (mt_off_t) This->cur_size,
 				  length);
 			if ( ret < 0 )
 				return ret;
-			This->cur_size += ret;
-			if (This->current+This->cur_size < start) {
+			This->cur_size += (size_t) ret;
+			if (This->current+(mt_off_t)This->cur_size < start) {
 				fprintf(stderr, "Short buffer fill\n");
 				exit(1);
 			}
@@ -168,14 +189,15 @@ static int buf_read(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 	disk_ptr = This->buf + offset;
 	maximize(len, This->cur_size - offset);
 	memcpy(buf, disk_ptr, len);
-	return len;
+	return (ssize_t) len;
 }
 
-static int buf_write(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
+static ssize_t buf_write(Stream_t *Stream, char *buf,
+			 mt_off_t start, size_t len)
 {
 	char *disk_ptr;
 	DeclareThis(Buffer_t);	
-	size_t offset;
+	size_t offset=0;
 
 	if(!len)
 		return 0;
@@ -183,10 +205,10 @@ static int buf_write(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 	This->ever_dirty = 1;
 
 #ifdef DEBUG
-	fprintf(stderr, "buf write %x   %02x %08x %08x -- %08x %08x -- %08x\n", 
+	fprintf(stderr, "buf write %x   %02x %08x %08x -- %08x %08x -- %08x\n",
 		Stream, (unsigned char) This->buf[0],
 		start, len, This->current, This->cur_size, This->size);
-	fprintf(stderr, "%d %d %d %x %x\n", 
+	fprintf(stderr, "%d %d %d %x %x\n",
 		start == This->current + This->cur_size,
 		This->cur_size < This->size,
 		len >= This->sectorSize, len, This->sectorSize);
@@ -196,27 +218,29 @@ static int buf_write(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 #ifdef DEBUG
 			fprintf(stderr, "outside\n");
 #endif
-			if(start % This->cylinderSize || 
+			if(start % (mt_off_t) This->cylinderSize ||
 			   len < This->sectorSize) {
 				size_t readSize;
-				int ret;
+				ssize_t ret;
+				size_t bytes_read;
+				
+				readSize = This->cylinderSize -
+					(size_t)(This->current % (mt_off_t) This->cylinderSize);
 
-				readSize = This->cylinderSize - 
-					This->current % This->cylinderSize;
-
-				ret=READS(This->Next, This->buf, This->current, readSize);
+				ret=READS(This->Next, This->buf, (mt_off_t)This->current, readSize);
 				/* read it! */
 				if ( ret < 0 )
 					return ret;
-				if(ret % This->sectorSize) {
-				    fprintf(stderr, "Weird: read size (%d) not a multiple of sector size (%d)\n", ret, This->sectorSize);
-				    ret -= ret % This->sectorSize;
-				    if(ret == 0) {
+				bytes_read = (size_t) ret;
+				if(bytes_read % This->sectorSize) {
+				  fprintf(stderr, "Weird: read size (%zd) not a multiple of sector size (%d)\n", bytes_read, (int) This->sectorSize);
+				    bytes_read -= bytes_read % This->sectorSize;
+				    if(bytes_read == 0) {
 					fprintf(stderr, "Nothing left\n");
 					exit(1);
 				    }
 				}
-				This->cur_size = ret;
+				This->cur_size = bytes_read;
 				/* for dosemu. Autoextend size */
 				if(!This->cur_size) {
 					memset(This->buf,0,readSize);
@@ -235,8 +259,7 @@ static int buf_write(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 			maximize(len, This->size - offset);
 			This->cur_size += len;
 			if(This->Next->Class->pre_allocate)
-				PRE_ALLOCATE(This->Next,
-							 This->current + This->cur_size);
+				PRE_ALLOCATE(This->Next, cur_end(This));
 			break;
 		case INSIDE:
 			/* nothing to do */
@@ -248,11 +271,11 @@ static int buf_write(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 			break;
 		case ERROR:
 			return -1;
-		default:
 #ifdef DEBUG
+		default:
 			fprintf(stderr, "Should not happen\n");
-#endif
 			exit(1);
+#endif
 	}
 
 	disk_ptr = This->buf + offset;
@@ -270,10 +293,12 @@ static int buf_write(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 		This->dirty_end = ROUND_UP(offset + len, This->sectorSize);
 	
 	if(This->dirty_end > This->cur_size) {
-		fprintf(stderr, 
+		fprintf(stderr,
 			"Internal error, dirty end too big dirty_end=%x cur_size=%x len=%x offset=%d sectorSize=%x\n",
-			This->dirty_end, (unsigned int) This->cur_size, (unsigned int) len, 
-				(int) offset, (int) This->sectorSize);
+			(unsigned int) This->dirty_end,
+			(unsigned int) This->cur_size,
+			(unsigned int) len,
+			(int) offset, (int) This->sectorSize);
 		fprintf(stderr, "offset + len + grain - 1 = %x\n",
 				(int) (offset + len + This->sectorSize - 1));
 		fprintf(stderr, "ROUNDOWN(offset + len + grain - 1) = %x\n",
@@ -284,7 +309,7 @@ static int buf_write(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 	}
 
 	This->dirty = 1;
-	return len;
+	return (ssize_t) len;
 }
 
 static int buf_flush(Stream_t *Stream)
@@ -319,15 +344,22 @@ static Class_t BufferClass = {
 	0, /* set_geom */
 	get_data_pass_through, /* get_data */
 	0, /* pre-allocate */
+	get_dosConvert_pass_through, /* dos convert */
+	0, /* discard */
 };
 
-Stream_t *buf_init(Stream_t *Next, int size, 
-		   int cylinderSize, 
-		   int sectorSize)
+Stream_t *buf_init(Stream_t *Next, size_t size,
+		   size_t cylinderSize,
+		   size_t sectorSize)
 {
 	Buffer_t *Buffer;
 	Stream_t *Stream;
 
+#ifdef HAVE_ASSERT_H
+	assert(size != 0);
+	assert(cylinderSize != 0);
+	assert(sectorSize != 0);
+#endif
 
 	if(size % cylinderSize != 0) {
 		fprintf(stderr, "size not multiple of cylinder size\n");
