@@ -262,6 +262,8 @@ static void get_env_conf(void)
 	s = getenv(global_switches[i].name);
 	if(s) {
 	    errno = 0;
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-align"
 	    switch(global_switches[i].type) {
 	    case T_INT:
 		* ((int *)global_switches[i].address) = strtoi(s,0,0);
@@ -280,6 +282,7 @@ static void get_env_conf(void)
 		* ((char **)global_switches[i].address) = s;
 		break;
 	    }
+#pragma GCC diagnostic pop
 	    if(errno != 0) {
 		fprintf(stderr, "Bad number %s for %s (%s)\n", s,
 			global_switches[i].name,
@@ -492,8 +495,6 @@ static void finish_drive_clause(void)
     }
     devices[cur_dev].file_nr = file_nr;
     devices[cur_dev].cfg_filename = filename;
-    if(! (flag_mask & PRIV_FLAG) && IS_SCSI(&devices[cur_dev]))
-	devices[cur_dev].misc_flags |= PRIV_FLAG;
     if(!trusted && (devices[cur_dev].misc_flags & PRIV_FLAG)) {
 	fprintf(stderr,
 		"Warning: privileged flag ignored for drive %c: defined in file %s\n",
@@ -511,6 +512,13 @@ static int set_var(struct switches_l *switches, int nr,
     for(i=0; i < nr; i++) {
 	if(match_token(switches[i].name)) {
 	    expect_char('=');
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-align"
+	    /* All pointers cast back to pointers with alignment
+	     * constraints were such pointers with alignment
+	     * constraints initially, thus they do indeed fit the
+	     * constraint */
+
 	    if(switches[i].type == T_UINT)
 		* ((unsigned int *)((long)switches[i].address+base_address)) =
 		    (unsigned int) get_unumber(UINT_MAX);
@@ -529,6 +537,7 @@ static int set_var(struct switches_l *switches, int nr,
 	    else if (switches[i].type == T_UQSTRING)
 		* ((char**)((long)switches[i].address+base_address))=
 		    get_unquoted_string();
+#pragma GCC diagnostic pop
 	    return 0;
 	}
     }
