@@ -30,7 +30,6 @@
 #include "mtools.h"
 #include "msdos.h"
 #include "scsi.h"
-#include "partition.h"
 #include "floppyd_io.h"
 
 #ifdef USE_FLOPPYD
@@ -38,10 +37,6 @@
 /* ######################################################################## */
 
 
-typedef unsigned char Byte;
-typedef unsigned long Dword;
-typedef mt_off_t Qword;
-
 static const char* AuthErrors[] = {
 	"Auth success",
 	"Auth failed: Packet oversized",
@@ -61,9 +56,9 @@ typedef struct RemoteFile_t {
 	int fd;
 	mt_off_t offset;
 	mt_off_t lastwhere;
-	mt_off_t size;
-	int version;
-	int capabilities;
+	mt_size_t size;
+	unsigned int version;
+	unsigned int capabilities;
 	int drive;
 } RemoteFile_t;
 
@@ -74,9 +69,12 @@ typedef struct RemoteFile_t {
 
 /* ######################################################################## */
 
-static int authenticate_to_floppyd(RemoteFile_t *floppyd, int sock, char *display)
+static unsigned int authenticate_to_floppyd(RemoteFile_t *floppyd,
+					    int sock, char *display)
 {
-	off_t filelen;
+	size_t cookielen;
+	uint16_t filelen;
+	ssize_t newlen;
 	Byte buf[16];
 	const char *command[] = { "xauth", "xauth", "extract", "-", 0, 0 };
 	char *xcookie;
@@ -85,21 +83,22 @@ static int authenticate_to_floppyd(RemoteFile_t *floppyd, int sock, char *displa
 
 	command[4] = display;
 
-	filelen=strlen(display);
-	filelen += 100;
+	cookielen=strlen(display);
+	cookielen += 100;
 
-	xcookie = (char *) safe_malloc(filelen+4);
-	filelen = safePopenOut(command, xcookie+4, filelen);
-	if(filelen < 1)
+	xcookie = (char *) safe_malloc(cookielen+4);
+	newlen = safePopenOut(command, xcookie+4, cookielen);
+	if(newlen < 1 || newlen > UINT16_MAX)
 		return AUTH_AUTHFAILED;
-
+	filelen = (uint16_t) newlen;
+	
 	/* Version negotiation */
 	dword2byte(4,buf);
 	dword2byte(floppyd->version,buf+4);
 	if(write(sock, buf, 8) < 8)
 		return AUTH_IO_ERROR;
 
-	if ( (l = read_dword(sock)) < 4) {
+	if ( (l = (int) read_dword(sock)) < 4) {
 		return AUTH_WRONGVERSION;
 	}
 
@@ -115,7 +114,7 @@ static int authenticate_to_floppyd(RemoteFile_t *floppyd, int sock, char *displa
 		floppyd->capabilities = read_dword(sock);
 
 	dword2byte(filelen, (Byte *)xcookie);
-	if(write(sock, xcookie, filelen+4) < filelen + 4)
+	if(write(sock, xcookie, filelen+4) < ((ssize_t) (filelen + 4)))
 		return AUTH_IO_ERROR;
 
 	if (read_dword(sock) != 4) {
@@ -128,11 +127,10 @@ static int authenticate_to_floppyd(RemoteFile_t *floppyd, int sock, char *displa
 }
 
 
-static int floppyd_reader(int fd, char* buffer, int len) 
+static ssize_t floppyd_reader(int fd, char* buffer, uint32_t len) 
 {
 	Dword errcode;
 	Dword gotlen;
-	int l;
 	Byte buf[16];
 
 	dword2byte(1, buf);
@@ -151,46 +149,54 @@ static int floppyd_reader(int fd, char* buffer, int len)
 	errcode = read_dword(fd);
 
 	if (gotlen != (Dword) -1) {
+		size_t l;
 		unsigned int start;
 		if (read_dword(fd) != gotlen) {
 			errno = EIO;
 			return -1;
 		}
 		for (start = 0, l = 0; start < gotlen; start += l) {
-			l = read(fd, buffer+start, gotlen-start);
-			if (l == 0) {
+			ssize_t ret = read(fd, buffer+start, gotlen-start);
+			if( ret < 0)
+				return -1;
+			if (ret == 0) {
 				errno = EIO;
 				return -1;
 			}
+			l = (size_t) ret;
 		}
 	} else {
-		errno = errcode;
+		errno = (int) errcode;
 	}
-	return gotlen;
+	return (ssize_t) gotlen;
 }
 
-static int floppyd_writer(int fd, char* buffer, int len) 
+static ssize_t floppyd_writer(int fd, char* buffer, uint32_t len) 
 {
-	Dword errcode;
-	Dword gotlen;
+	int errcode;
+	int32_t gotlen;
 	Byte buf[16];
+	ssize_t ret;
 
 	dword2byte(1, buf);
 	buf[4] = OP_WRITE;
 	dword2byte(len, buf+5);
 
+	cork(fd, 1);
 	if(write(fd, buf, 9) < 9)
 		return AUTH_IO_ERROR;
-	if(write(fd, buffer, len) < len)
+	ret = write(fd, buffer, len);
+	if(ret == -1 || (size_t) ret < len)
 		return AUTH_IO_ERROR;
+	cork(fd, 0);
 	
 	if (read_dword(fd) != 8) {
 		errno = EIO;
 		return -1;
 	}
 
-	gotlen = read_dword(fd);
-	errcode = read_dword(fd);
+	gotlen = read_sdword(fd);
+	errcode = read_sdword(fd);
 
 	errno = errcode;
 	if(errno != 0 && gotlen == 0) {
@@ -202,18 +208,18 @@ static int floppyd_writer(int fd, char* buffer, int len)
 	return gotlen;
 }
 
-static int floppyd_lseek(int fd, mt_off_t offset, int whence) 
+static int floppyd_lseek(int fd, int32_t offset, int whence) 
 {
-	Dword errcode;
-	Dword gotlen;
+	int errcode;
+	int gotlen;
 	Byte buf[32];
 	
 	dword2byte(1, buf);
 	buf[4] = OP_SEEK;
 	
 	dword2byte(8, buf+5);
-	dword2byte(truncBytes32(offset), buf+9);
-	dword2byte(whence, buf+13);
+	sdword2byte(offset, buf+9);
+	sdword2byte(whence, buf+13);
 	
 	if(write(fd, buf, 17) < 17)
 		return AUTH_IO_ERROR;
@@ -223,8 +229,8 @@ static int floppyd_lseek(int fd, mt_off_t offset, int whence)
 		return -1;
 	}
 
-	gotlen = read_dword(fd);
-	errcode = read_dword(fd);
+	gotlen = read_sdword(fd);
+	errcode = read_sdword(fd);
 
 	errno = errcode;
 	
@@ -233,16 +239,16 @@ static int floppyd_lseek(int fd, mt_off_t offset, int whence)
 
 static mt_off_t floppyd_lseek64(int fd, mt_off_t offset, int whence) 
 {
-	Dword errcode;
-	Qword gotlen;
+	int errcode;
+	struct SQwordRet gotlen;
 	Byte buf[32];
 	
 	dword2byte(1, buf);
 	buf[4] = OP_SEEK64;
 	
 	dword2byte(12, buf+5);
-	qword2byte(offset, buf+9);
-	dword2byte(whence, buf+17);
+	qword2byte((uint32_t)offset, buf+9);
+	sdword2byte(whence, buf+17);
 	
 	if(write(fd, buf, 21) < 21)
 		return AUTH_IO_ERROR;
@@ -252,18 +258,18 @@ static mt_off_t floppyd_lseek64(int fd, mt_off_t offset, int whence)
 		return -1;
 	}
 
-	gotlen = read_qword(fd);
-	errcode = read_dword(fd);
+	gotlen = read_sqword(fd);
+	errcode = read_sdword(fd);
 
 	errno = errcode;
 	
-	return gotlen;
+	return gotlen.v;
 }
 
 static int floppyd_open(RemoteFile_t *This, int mode) 
 {
-	Dword errcode;
-	Dword gotlen;
+	int errcode;
+	int gotlen;
 	Byte buf[16];
 	
 	if(! (This->capabilities & FLOPPYD_CAP_EXPLICIT_OPEN) ) {
@@ -277,7 +283,7 @@ static int floppyd_open(RemoteFile_t *This, int mode)
 	else
 		buf[4] = OP_OPRW;
 	dword2byte(4, buf+5);
-	dword2byte(This->drive, buf+9);
+	sdword2byte(This->drive, buf+9);
 
 	if(write(This->fd, buf, 13) < 13)
 		return AUTH_IO_ERROR;
@@ -287,8 +293,8 @@ static int floppyd_open(RemoteFile_t *This, int mode)
 		return -1;
 	}
 
-	gotlen = read_dword(This->fd);
-	errcode = read_dword(This->fd);
+	gotlen = read_sdword(This->fd);
+	errcode = read_sdword(This->fd);
 
 	errno = errcode;
 	
@@ -298,13 +304,13 @@ static int floppyd_open(RemoteFile_t *This, int mode)
 
 /* ######################################################################## */
 
-typedef int (*iofn) (int, char *, int);
+typedef ssize_t (*iofn) (int, char *, uint32_t);
 
-static int floppyd_io(Stream_t *Stream, char *buf, mt_off_t where, int len,
-		   iofn io)
+static ssize_t floppyd_io(Stream_t *Stream, char *buf, mt_off_t where,
+			  size_t len, iofn io)
 {
 	DeclareThis(RemoteFile_t);
-	int ret;
+	ssize_t ret;
 
 	where += This->offset;
 
@@ -316,31 +322,38 @@ static int floppyd_io(Stream_t *Stream, char *buf, mt_off_t where, int len,
 				return -1;
 			}
 		} else {
-			if(floppyd_lseek( This->fd, where, SEEK_SET) < 0 ){
+			if(where > INT32_MAX  || where < INT32_MIN) {
+				fprintf(stderr, "Seek position out of range\n");
+				return -1;
+			}
+			if(floppyd_lseek(This->fd, (int32_t) where, SEEK_SET) < 0 ){
 				perror("floppyd_lseek");
 				This->lastwhere = (mt_off_t) -1;
 				return -1;
 			}
 		}
 	}
-	ret = io(This->fd, buf, len);
+	ret = io(This->fd, buf,
+		 (len > INT32_MAX) ? (uint32_t)INT32_MAX+1 : (uint32_t) len);
 	if ( ret == -1 ){
 		perror("floppyd_io");
 		This->lastwhere = (mt_off_t) -1;
 		return -1;
 	}
-	This->lastwhere = where + ret;
+	This->lastwhere = where + (mt_off_t) ret;
 	return ret;
 }
 
-static int floppyd_read(Stream_t *Stream, char *buf, mt_off_t where, size_t len)
+static ssize_t floppyd_read(Stream_t *Stream, char *buf,
+			    mt_off_t where, size_t len)
 {	
-	return floppyd_io(Stream, buf, where, len, (iofn) floppyd_reader);
+	return floppyd_io(Stream, buf, where, len, floppyd_reader);
 }
 
-static int floppyd_write(Stream_t *Stream, char *buf, mt_off_t where, size_t len)
+static ssize_t floppyd_write(Stream_t *Stream, char *buf,
+			     mt_off_t where, size_t len)
 {
-	return floppyd_io(Stream, buf, where, len, (iofn) floppyd_writer);
+	return floppyd_io(Stream, buf, where, len, floppyd_writer);
 }
 
 static int floppyd_flush(Stream_t *Stream)
@@ -385,8 +398,8 @@ static int floppyd_free(Stream_t *Stream)
 		    return -1;
 		}
 		
-		gotlen = read_dword(This->fd);
-		errcode = read_dword(This->fd);
+		gotlen = read_sdword(This->fd);
+		errcode = read_sdword(This->fd);
 		
 		errno = errcode;
 
@@ -397,38 +410,10 @@ static int floppyd_free(Stream_t *Stream)
 	}
 }
 
-static int floppyd_geom(Stream_t *Stream, struct device *dev, 
-			struct device *orig_dev UNUSEDP,
-			int media, union bootsector *boot)
-{
-	size_t tot_sectors;
-	int sect_per_track;
-	DeclareThis(RemoteFile_t);
-
-	dev->ssize = 2; /* allow for init_geom to change it */
-	dev->use_2m = 0x80; /* disable 2m mode to begin */
-
-	if(media == 0xf0 || media >= 0x100){		
-		dev->heads = WORD(nheads);
-		dev->sectors = WORD(nsect);
-		tot_sectors = DWORD(bigsect);
-		SET_INT(tot_sectors, WORD(psect));
-		sect_per_track = dev->heads * dev->sectors;
-		tot_sectors += sect_per_track - 1; /* round size up */
-		dev->tracks = tot_sectors / sect_per_track;
-
-	} else
-		if(setDeviceFromOldDos(media, dev) < 0)
-			exit(1);
-
-	This->size = (mt_off_t) 512 * dev->sectors * dev->tracks * dev->heads;
-
-	return 0;
-}
 
 
 static int floppyd_data(Stream_t *Stream, time_t *date, mt_size_t *size,
-		     int *type, int *address)
+			int *type, uint32_t *address)
 {
 	DeclareThis(RemoteFile_t);
 
@@ -437,7 +422,7 @@ static int floppyd_data(Stream_t *Stream, time_t *date, mt_size_t *size,
 		*date = 0;
 	if(size)
 		/* the size derived from the geometry */
-		*size = (mt_size_t) This->size;
+		*size = This->size;
 	if(type)
 		*type = 0; /* not a directory */
 	if(address)
@@ -452,7 +437,7 @@ static Class_t FloppydFileClass = {
 	floppyd_write,
 	floppyd_flush,
 	floppyd_free,
-	floppyd_geom,
+	set_geom_noop,
 	floppyd_data,
 	0, /* pre_allocate */
 	0, /* get_dosConvert */
@@ -462,7 +447,8 @@ static Class_t FloppydFileClass = {
 /* ######################################################################## */
 
 static int get_host_and_port_and_drive(const char* name, char** hostname,
-				       char **display, short* port, int *drive)
+				       char **display, uint16_t* port,
+				       int *drive)
 {
 	char* newname = strdup(name);
 	char* p;
@@ -476,12 +462,12 @@ static int get_host_and_port_and_drive(const char* name, char** hostname,
 	
 	*port = FLOPPYD_DEFAULT_PORT;	
 	if(*p >= '0' && *p <= '9')
-	  *port = strtoul(p, &p, 0);
+	  *port = strtou16(p, &p, 0);
 	if(*p == '/')
 	  p++;
 	*drive = 0;
 	if(*p >= '0' && *p <= '9')
-	  *drive = strtoul(p, &p, 0);
+	  *drive = strtoi(p, &p, 0);
 
 	*display = strdup(newname);
 
@@ -505,11 +491,10 @@ static int get_host_and_port_and_drive(const char* name, char** hostname,
 /*
  *  * Return the IP address of the specified host.
  *  */
-static IPaddr_t getipaddress(char *ipaddr)
+static in_addr_t getipaddress(char *ipaddr)
 {
-	
 	struct hostent  *host;
-	IPaddr_t        ip;
+	in_addr_t        ip;
 
 	if (((ip = inet_addr(ipaddr)) == INADDR_NONE) &&
 	    (strcmp(ipaddr, "255.255.255.255") != 0)) {
@@ -531,7 +516,7 @@ static IPaddr_t getipaddress(char *ipaddr)
 /*
  *  * Connect to the floppyd server.
  *  */
-static int connect_to_server(IPaddr_t ip, short port)
+static int connect_to_server(in_addr_t ip, uint16_t port)
 {
 	
 	struct sockaddr_in      addr;
@@ -575,7 +560,7 @@ static int ConnectToFloppyd(RemoteFile_t *floppyd, const char* name,
 			    char *errmsg);
 
 Stream_t *FloppydOpen(struct device *dev, 
-		      char *name, int mode, char *errmsg,
+		      const char *name, int mode, char *errmsg,
 		      mt_size_t *maxSize)
 {
 	RemoteFile_t *This;
@@ -610,9 +595,9 @@ Stream_t *FloppydOpen(struct device *dev,
 	}
 
 	if(maxSize) {
-		*maxSize = 
-			(This->capabilities & FLOPPYD_CAP_LARGE_SEEK) ?
-			max_off_t_seek : max_off_t_31;
+		*maxSize = (mt_size_t)
+			((This->capabilities & FLOPPYD_CAP_LARGE_SEEK) ?
+			 max_off_t_seek : max_off_t_31);
 	}
 	return (Stream_t *) This;
 }
@@ -622,11 +607,11 @@ static int ConnectToFloppyd(RemoteFile_t *floppyd, const char* name,
 {
 	char* hostname;
 	char* display;
-	short port;
+	uint16_t port;
 	int rval = get_host_and_port_and_drive(name, &hostname, &display, 
 					       &port, &floppyd->drive);
 	int sock;
-	int reply;
+	unsigned int reply;
 	
 	if (!rval) return -1;
 
