@@ -1,15 +1,23 @@
-/* Test program for doctoring the fat */
-
-
-/*
- * mcopy.c
- * Copy an MSDOS files to and from Unix
+/*  Copyright 1999,2001,2002,2009 Alain Knaff.
+ *  This file is part of mtools.
  *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Test program for doctoring the fat
  */
 
 
-#define LOWERCASE
-
 #include "sysincludes.h"
 #include "msdos.h"
 #include "mtools.h"
@@ -26,10 +34,10 @@ typedef struct Arg_t {
 	MainParam_t mp;
 	ClashHandling_t ch;
 	Stream_t *sourcefile;
-	unsigned long fat;
+	uint32_t fat;
 	int markbad;
 	int setsize;
-	unsigned long size;
+	uint32_t size;
 	Fs_t *Fs;
 } Arg_t;
 
@@ -50,30 +58,32 @@ static int dos_doctorfat(direntry_t *entry, MainParam_t *mp)
 	return GOT_ONE;
 }
 
-static int unix_doctorfat(MainParam_t *mp)
+static int unix_doctorfat(MainParam_t *mp UNUSEDP)
 {
 	fprintf(stderr,"File does not reside on a Dos fs\n");
 	return ERROR_ONE;
 }
 
-
-static void usage(void)
+static void usage(int ret) NORETURN;
+static void usage(int ret)
 {
 	fprintf(stderr,
 		"Mtools version %s, dated %s\n", mversion, mdate);
 	fprintf(stderr,
 		"Usage: [-b] %s file fat\n", progname);
-	exit(1);
+	exit(ret);
 }
 
-void mdoctorfat(int argc, char **argv, int mtype)
+void mdoctorfat(int argc, char **argv, int mtype UNUSEDP) NORETURN;
+void mdoctorfat(int argc, char **argv, int mtype UNUSEDP)
 {
 	Arg_t arg;
 	int c, ret;
-	long address, begin, end;
+	unsigned int address;
+	unsigned int begin, end;
 	char *number, *eptr;
-	int i, j;
-	long offset;
+	int i;
+	unsigned int offset;
 	
 	/* get command line options */
 
@@ -85,29 +95,35 @@ void mdoctorfat(int argc, char **argv, int mtype)
 	arg.setsize = 0;
 
 	/* get command line options */
-	while ((c = getopt(argc, argv, "i:bo:s:")) != EOF) {
+	if(helpFlag(argc, argv))
+		usage(0);
+	while ((c = getopt(argc, argv, "i:bo:s:h")) != EOF) {
+		char *endptr = NULL;
+		errno=0;
 		switch (c) {
 			case 'i':
-				set_cmd_line_image(optarg, 0);
+				set_cmd_line_image(optarg);
 				break;
 			case 'b':
 				arg.markbad = 1;
 				break;
 			case 'o':
-				offset = strtoul(optarg,0,0);
+				offset = strtoui(optarg,&endptr,0);
 				break;
 			case 's':
 				arg.setsize=1;
-				arg.size = strtoul(optarg,0,0);
+				arg.size = strtou32(optarg,&endptr,0);
 				break;
+			case 'h':
+				usage(0);
 			case '?':
-				usage();
-				break;
+				usage(1);
 		}
+		check_number_parse_errno((char)c, optarg, endptr);
 	}
 
 	if (argc - optind < 2)
-		usage();
+		usage(1);
 
 
 	/* only 1 file to copy... */
@@ -119,20 +135,21 @@ void mdoctorfat(int argc, char **argv, int mtype)
 	
 	arg.mp.lookupflags = ACCEPT_PLAIN | ACCEPT_DIR | DO_OPEN;
 	arg.mp.openflags = O_RDWR;
-	arg.fat = strtoul(argv[optind+1], 0, 0) + offset;
+	arg.fat = strtoui(argv[optind+1], 0, 0) + offset;
 	ret=main_loop(&arg.mp, argv + optind, 1);
 	if(ret)
 		exit(ret);
 	address = 0;
 	for(i=optind+1; i < argc; i++) {
+		unsigned int j;
 		number = argv[i];
 		if (*number == '<') {
 			number++;
 		}
-		begin = strtoul(number, &eptr, 0);
+		begin = strtoui(number, &eptr, 0);
 		if (eptr && *eptr == '-') {
 			number = eptr+1;
-			end = strtoul(number, &eptr, 0);
+			end = strtoui(number, &eptr, 0);
 		} else {
 			end = begin;
 		}
