@@ -30,8 +30,8 @@
 static void _label_name(doscp_t *cp, const char *filename, int verbose UNUSEDP,
 			int *mangled, dos_name_t *ans, int preserve_case)
 {
-	int len;
-	int i;
+	size_t len;
+	size_t i;
 	int have_lower, have_upper;
 	wchar_t wbuffer[12];
 
@@ -51,7 +51,7 @@ static void _label_name(doscp_t *cp, const char *filename, int verbose UNUSEDP,
 		if(isupper(wbuffer[i]))
 			have_upper = 1;
 		if(!preserve_case)
-			wbuffer[i] = towupper(wbuffer[i]);
+			wbuffer[i] = ch_towupper(wbuffer[i]);
 		if(
 #ifdef HAVE_WCHAR_H
 		   wcschr(L"^+=/[]:,?*\\<>|\".", wbuffer[i])
@@ -119,7 +119,7 @@ void mlabel(int argc, char **argv, int type UNUSEDP)
 	int c;
 	int mangled;
 	enum { SER_NONE, SER_RANDOM, SER_SET }  set_serial = SER_NONE;
-	long serial = 0;
+	uint32_t serial = 0;
 	int need_write_boot = 0;
 	int have_boot = 0;
 	char *eptr;
@@ -158,18 +158,20 @@ void mlabel(int argc, char **argv, int type UNUSEDP)
 				break;
 			case 'n':
 				set_serial = SER_RANDOM;
-				srandom(time (0));
-				serial=random();
+				init_random();
+				serial=(uint32_t) random();
 				break;
 			case 'N':
 				set_serial = SER_SET;
-				serial = strtoul(optarg, &eptr, 16);
+				errno=0;
+				serial = strtou32(optarg, &eptr, 16);
 				if(*eptr) {
 					fprintf(stderr,
 						"%s not a valid serial number\n",
 						optarg);
 					exit(1);
 				}
+				check_number_parse_errno((char)c, optarg, eptr);
 				break;
 			case 'h':
 				usage(0);
@@ -183,7 +185,7 @@ void mlabel(int argc, char **argv, int type UNUSEDP)
 	if(argc - optind == 1) {
 	    if(!argv[optind][0] || argv[optind][1] != ':')
 		usage(1);
-	    drive = toupper(argv[argc -1][0]);
+	    drive = ch_toupper(argv[argc -1][0]);
 	    newLabel = argv[optind]+2;
 	} else {
 	    drive = get_default_drive();
@@ -219,7 +221,8 @@ void mlabel(int argc, char **argv, int type UNUSEDP)
 
 	initializeDirentry(&entry, RootDir);
 	r=vfat_lookup(&entry, 0, 0, ACCEPT_LABEL | MATCH_ANY,
-		      shortname, longname);
+		      shortname, sizeof(shortname),
+		      longname, sizeof(longname));
 	if (r == -2) {
 		FREE(&RootDir);
 		exit(1);
@@ -302,7 +305,8 @@ void mlabel(int argc, char **argv, int type UNUSEDP)
 		label_name_pc(cp, shrtLabel, verbose, &mangled, &dosname);
 
 		if(have_boot && boot.boot.descr >= 0xf0 && has_BPB4) {
-			strncpy(labelBlock->label, dosname.base, 11);
+			strncpy(labelBlock->label, dosname.base, 8);
+			strncpy(labelBlock->label+8, dosname.ext, 3);
 			need_write_boot = 1;
 
 		}
