@@ -20,8 +20,6 @@
  */
 
 
-#define LOWERCASE
-
 #include "sysincludes.h"
 #include "msdos.h"
 #include "mtools.h"
@@ -65,7 +63,7 @@ static int makeit(dos_name_t *dosname,
 {
 	Stream_t *Target;
 	CreateArg_t *arg = (CreateArg_t *) arg0;
-	int fat;
+	uint32_t fat;
 	direntry_t subEntry;	
 
 	/* will it fit? At least one cluster must be free */
@@ -153,6 +151,7 @@ static int createDirCallback(direntry_t *entry UNUSEDP, MainParam_t *mp)
 	
 }
 
+void mmd(int argc, char **argv, int type UNUSEDP) NORETURN;
 void mmd(int argc, char **argv, int type UNUSEDP)
 {
 	Arg_t arg;
@@ -173,7 +172,7 @@ void mmd(int argc, char **argv, int type UNUSEDP)
 			case '?':
 				usage(1);
 			case 'o':
-				handle_clash_options(&arg.ch, c);
+				handle_clash_options(&arg.ch, (char) c);
 				break;
 			case 'D':
 				if(handle_clash_options(&arg.ch, *optarg))
@@ -183,7 +182,6 @@ void mmd(int argc, char **argv, int type UNUSEDP)
 				usage(0);
 			default:
 				usage(1);
-				break;
 		}
 	}
 
